from distutils.core import setup

setup(
    name='RainbowChain',
    packages=['RainbowChain'],
    license="MIT License",
    version='0.1.4',
    description='Implementing Middle Out (Indexing) to the Markov Chain',
    author='Avery Durrant',
    author_email='avery@frostbyte.co',
    url='https://github.com/Avery246813579/Python-Rainbow-Chain',
    download_url='https://github.com/Avery246813579/Python-Rainbow-Chain/0.1.4',
    keywords=['Markov Chain', 'Middle Out', 'Markov Model'],
    classifiers=[]
)
